# SPDX-FileCopyrightText: 2024-present gunungpw <gunungpambudiw@gmail.com>
#
# SPDX-License-Identifier: MIT

from .name import (
    rgb_to_name,
    hex_to_name,
    hsl_to_name,
    lab_to_name,
)

from .colors import NamedColor
from .data import get_data

__all__ = [
    "NamedColor",
    "rgb_to_name",
    "hex_to_name",
    "hsl_to_name",
    "lab_to_name",
]

get_data.download_json_data()



# color.py

from . import name


class NamedColor:
    """
    Represents a color with a name from a specified color database.

    Attributes:
        system: The color system ('RGB', 'HSL', 'LAB').
        coordinate: The coordinates representing the color.
        value_hex: The hexadecimal representation of the color value.
    """

    def __init__(self, input: tuple[str, int, int, int]) -> None:
        """
        Initializes a NamedColor instance.

        Args:
            input: A tuple containing the color system and its coordinates.
        """
        self.system, *self.coordinate = input
        self.value_hex: str | None = None
        if self.system.upper() in ("RGB", "HSL", "LAB"):
            if self.system == "RGB":
                self.value_hex = rgb_to_hex(self.coordinate)
            elif self.system == "HSL":
                self.value_hex = rgb_to_hex(self.coordinate)
            elif self.system == "LAB":
                self.value_hex = rgb_to_hex(self.coordinate)
            else:
                raise NotImplementedError

    @property
    def hex(self) -> str | None:
        """
        Gets the hexadecimal representation of the color value.

        Returns:
            str | None: The hexadecimal color value or None if not available.
        """
        return self.value_hex

    @property
    def name(self) -> str | None:
        """
        Gets the name of the color based on its hexadecimal representation.

        Returns:
            str: The name of the color if found, otherwise a message indicating
            that the name is not found.
        """
        if self.value_hex is None:
            return f"{self.value_hex} name is not found"
        return name.hex_to_name(self.value_hex)

    def __str__(self) -> str:
        """
        Returns a string representation of the NamedColor instance.

        Returns:
            str: A string representation of the NamedColor.
        """
        return f"{self.system}, {self.coordinate}"


def rgb_to_hex(rgb) -> str | None:
    return "#{:02x}{:02x}{:02x}".format(*rgb)



import json
import os
from typing import TypeAlias, Literal

data_path = os.path.dirname(__file__)

ColorDB: TypeAlias = Literal[
    "default",
    "bestOf",
    "wikipedia",
    "french",
    "spanish",
    "german",
    "ridgway",
    "risograph",
    "basic",
    "chineseTraditional",
    "html",
    "japaneseTraditional",
    "leCorbusier",
    "nbsIscc",
    "ntc",
    "osxcrayons",
    "ral",
    "sanzoWadaI",
    "thesaurus",
    "werner",
    "windows",
    "x11",
    "xkcd",
]


def hex_to_name(hex: str, color_db: ColorDB = "default") -> str | None:
    """Converts a hexadecimal color representation to its corresponding color name.

    Args:
        hex (str): The hexadecimal color representation.
        color_db (ColorDB, optional): The color database to use. Defaults to "default".

    Returns:
        str | None: The corresponding color name if found, otherwise None.
    """
    with open(f"{data_path}\\data\\{color_db}.json", "r") as f:
        color_name = json.load(f)

    for color in color_name["colors"]:
        if color["hex"] == hex.lower():
            return color["name"]
    return None


def rgb_to_name(rgb: tuple[int, int, int], color_db: ColorDB = "default") -> str | None:
    """Converts an RGB color representation to its corresponding color name.

    Args:
        rgb (tuple[int, int, int]): The RGB color representation as a tuple of integers (R, G, B).
        color_db (ColorDB, optional): The color database to use. Defaults to "default".

    Returns:
        str | None: The corresponding color name if found, otherwise None.
    """

    with open(f"{data_path}\\data\\{color_db}.json", "r") as f:
        color_name = json.load(f)

    for color in color_name["colors"]:
        if (color["rgb"]["r"], color["rgb"]["g"], color["rgb"]["b"]) == rgb:
            return color["name"]
    return None


def hsl_to_name(hsl: tuple[int, int, int], color_db: ColorDB = "default") -> str | None:
    """Converts an HSL color representation to its corresponding color name.

    Args:
        hsl (tuple[int, int, int]): The HSL color representation as a tuple of integers (H, S, L).
        color_db (ColorDB, optional): The color database to use. Defaults to "default".

    Returns:
        str | None: The corresponding color name if found, otherwise None.
    """
    with open(f"{data_path}\\data\\{color_db}.json", "r") as f:
        color_name = json.load(f)

    for color in color_name["colors"]:
        if (color["hsl"]["h"], color["hsl"]["s"], color["hsl"]["l"]) == hsl:
            return color["name"]
    return None


def lab_to_name(lab: tuple[int, int, int], color_db: ColorDB = "default") -> str | None:
    """Converts a LAB color representation to its corresponding color name.

    Args:
        lab (tuple[int, int, int]): The LAB color representation as a tuple of integers (L, A, B).
        color_db (ColorDB, optional): The color database to use. Defaults to "default".

    Returns:
        str | None: The corresponding color name if found, otherwise None.
    """
    with open(f"{data_path}\\data\\{color_db}.json", "r") as f:
        color_name = json.load(f)

    for color in color_name["colors"]:
        if (color["lab"]["l"], color["lab"]["a"], color["lab"]["b"]) == lab:
            return color["name"]
    return None



import json
import os
import urllib3
from concurrent.futures import ThreadPoolExecutor


url_source = [
    "default",
    "bestOf",
    "wikipedia",
    "french",
    "spanish",
    "german",
    "ridgway",
    "risograph",
    "basic",
    "chineseTraditional",
    "html",
    "japaneseTraditional",
    "leCorbusier",
    "nbsIscc",
    "ntc",
    "osxcrayons",
    "ral",
    "sanzoWadaI",
    "thesaurus",
    "werner",
    "windows",
    "x11",
    "xkcd",
]

data_dir = os.path.dirname(__file__)


def check_file() -> bool:
    check: list[bool] = []
    for name in url_source:
        file_path = os.path.join(data_dir, f"{name}.json")
        if os.path.isfile(file_path) is False:
            check.append(False)
    return all(check)


def download_json_data():
    if check_file() is False:
        print("Colors database file not exist.\n Download data:")
        http = urllib3.PoolManager()

        def download_file(file_name):
            print(f"Download {file_name}.json...")
            file_path = os.path.join(data_dir, f"{file_name}.json")
            with open(file_path, "w") as file:
                url = f"https://api.color.pizza/v1/?list={file_name}"
                response = http.request("GET", url)
                json.dump(response.json(), file)

        with ThreadPoolExecutor() as executor:
            executor.map(download_file, url_source)
